--
-- WorkRPM
-- Increase motor RPM when using hydraulics tools (moving tools) and show RPM Hud.
-- @author:    	kenny456 (kenny456@seznam.cz) / Xentro (Marcus@Xentro.se) / Webby / [FSM]Chefkoch
-- @history:	v1.0 - 2017-11-06 - 
--				v1.1 - 2017-11-10 - fixed slowning down vehicle when running with implement turned on
--				v1.2 - 2017-12-02 - -added increase RPM when tipping
--									-added increase RPM when folding / unfolding implement / trailer
--									-added increase RPM to combines tipping and folding / unfolding pipe
--									-added increase RPM when moving movingTools of attached implements / trailers
--									-fixed slowning down machine when running with implement turned on or moving with movingTools
--
WorkRPM = {};
local WorkRPM_directory = g_currentModDirectory;
function WorkRPM.prerequisitesPresent(specializations)
     return SpecializationUtil.hasSpecialization(Motorized, specializations);
end;

function WorkRPM:load(savegame)
	self.getPtoRpm = Utils.overwrittenFunction(self.getPtoRpm, WorkRPM.getPtoRpm);
	self.getDoConsumePtoPower = Utils.overwrittenFunction(self.getDoConsumePtoPower, WorkRPM.getDoConsumePtoPower);
	
	self.workRpm = {};
	self.workRpm.rpm = 0;
	self.workRpm.releaseRpmTimer = 0;
	self.workRpm.maxTime = 50; -- how long rpm last after movingTool stopped
	self.workRpm.minRpm = Utils.getNoNil(getXMLInt(self.xmlFile, "vehicle.motor#minRpm"), 1000) / 3;
	self.workRpm.maxRpm = Utils.getNoNil(getXMLInt(self.xmlFile, "vehicle.motor#maxRpm"), 2200) / 3;
	local Rpm = getXMLFloat(self.xmlFile, "vehicle.WorkRPM");
	if Rpm ~= nil then
		self.workRpm.rpmOnLoad = Rpm / 4;
		self.workRpm.rpmOnTipping = Rpm / 4;
		self.workRpm.rpmTurnedOn = Rpm / 4;
		self.workRpm.rpmFolding = Rpm / 4;
	else
		self.workRpm.rpmOnLoad = self.workRpm.minRpm * 1.7; -- how high rpm will go when movingTool move -- 400 / 1600
		self.workRpm.rpmOnTipping = self.workRpm.minRpm * 1.7; -- how high rpm will go when Tipping -- 450 / 1800
		self.workRpm.rpmTurnedOn = self.workRpm.minRpm * 1.7; -- how high rpm will go when is TurnedOn -- 500 / 2000
		self.workRpm.rpmFolding = self.workRpm.minRpm * 1.3; -- how high rpm will go when Combine is TurnedOn -- 475 / 1900
	end;
	self.workRpm.movingToolIds = {1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16}; -- movingTool indice
	self.workRpm.showRPMPosX = 0.868;
    self.workRpm.showRPMWidth = 0.012;
    self.workRpm.showRPMPosY = 0.072;
    self.workRpm.showRPMHeight = 0.02;
	self.workRpm.showRPMHudOverlay = Overlay:new("RPMHUDOverlay", Utils.getFilename("rpmHud2.dds", WorkRPM_directory), self.workRpm.showRPMPosX, self.workRpm.showRPMPosY, self.workRpm.showRPMWidth, self.workRpm.showRPMHeight);
	self.workRpm.isFoldable = false
	self.workRpm.foldableAnim = nil
	self.workRpm.isTurnedOn = false;
	
	local filename = getXMLString(self.xmlFile, "vehicle.filename");
	self.isTipScrollerVehicle = hasXMLProperty(self.xmlFile, "vehicle.tipScrollerNodes");
	self.workRpm.isOverloadVehicle = SpecializationUtil.hasSpecialization(Overloading, self.specializations) or string.find(filename, "flieglOverloadStation") or false;
	if string.find(self.configFileName, "pdlc/itRunnerPack") and g_modNameToDirectory["pdlc_itRunnerPack"] then
		self.isItRunnerHookLiftTrailerModul = filename == "itRunner/containerSilage.i3d" or filename == "itRunner/containerDumper.i3d";
		self.isItRunnerHookLiftTrailer = filename == "itRunner/itRunner2633HD.i3d";
	else
		self.isItRunnerHookLiftTrailerModul = false;
		self.isItRunnerHookLiftTrailer = false;
	end;
	if SpecializationUtil.hasSpecialization(Foldable, self.specializations) then
			local anim = Utils.getNoNil(getXMLString(self.xmlFile, "vehicle.foldingParts.foldingPart#animationName"), "folding");
			if anim ~= nil then
				self.workRpm.isFoldable = true
				self.workRpm.foldableAnim = anim
			end
	end
	
	self.workRpm.filename = Utils.getNoNil(getXMLString(self.xmlFile, "vehicle.filename"),"NENI");

end;

function WorkRPM:delete()
end;

function WorkRPM:readStream(streamId, connection)
end;

function WorkRPM:writeStream(streamId, connection)
end;

function WorkRPM:mouseEvent(posX, posY, isDown, isUp, button)
end;

function WorkRPM:keyEvent(unicode, sym, modifier, isDown)
end;

function WorkRPM:update(dt)
end;

function WorkRPM:updateTick(dt)
	if self:getIsActive() then
		local neededRpm = 0;
		--		Implement TurnedOn		--
		if SpecializationUtil.hasSpecialization(TurnOnVehicle, self.specializations) then
			if self:getIsTurnedOn() then
				if not SpecializationUtil.hasSpecialization(WoodHarvester, self.specializations) then
					if SpecializationUtil.hasSpecialization(Combine, self.specializations) then
						neededRpm = self.workRpm.rpmTurnedOn
						self.workRpm.isTurnedOn = true
					else
						neededRpm = self.workRpm.rpmTurnedOn
						self.workRpm.isTurnedOn = true
					end
				end
			else
				self.workRpm.isTurnedOn = false
			end
		end
		--		Foldable		--
		if neededRpm == 0 then
			if SpecializationUtil.hasSpecialization(Foldable, self.specializations) then
				if self.workRpm.isFoldable == true and self.workRpm.foldableAnim ~= nil then
					if self:getIsAnimationPlaying(self.workRpm.foldableAnim) then
						neededRpm = self.workRpm.rpmOnTipping;
					end
				end
			end
		end
				--		Tipping		--
		if neededRpm == 0 then
			local trailer;
			for _, attachedTrailer in pairs(self.attachedImplements) do
				if attachedTrailer.object ~= nil then
					trailer = attachedTrailer.object;
					if trailer.isTipScrollerVehicle or trailer.isItRunnerHookLiftTrailerModul then
						if trailer.tipState == Trailer.TIPSTATE_OPENING or trailer.tipState == Trailer.TIPSTATE_OPEN then
							neededRpm = self.workRpm.rpmOnTipping;
						end
					elseif trailer.isItRunnerHookLiftTrailer then
						if trailer:getIsAnimationPlaying("unfoldHand") then
							neededRpm = self.workRpm.rpmOnTipping;
						end
					elseif trailer.workRpm.isOverloadVehicle then
						if trailer:getOverloadingActive() then
							neededRpm = self.workRpm.rpmOnTipping;
						end
					else
						if trailer.tipState == Trailer.TIPSTATE_OPENING then
							neededRpm = self.workRpm.rpmOnTipping;
						end
					end;
				end;
			end;
			if SpecializationUtil.hasSpecialization(Drivable, self.specializations) and self.isMotorStarted then
				if trailer ~= nil then
					if trailer.isTipScrollerVehicle or trailer.isItRunnerHookLiftTrailerModul then
						if self.isTipScrollerVehicle then
							if self.tipState == Trailer.TIPSTATE_OPENING or self.tipState == Trailer.TIPSTATE_OPEN or trailer.tipState == Trailer.TIPSTATE_OPENING or trailer.tipState == Trailer.TIPSTATE_OPEN then
								neededRpm = self.workRpm.rpmOnTipping;
							end
						else
							if self.tipState == Trailer.TIPSTATE_OPENING or trailer.tipState == Trailer.TIPSTATE_OPENING or trailer.tipState == Trailer.TIPSTATE_OPEN then
								neededRpm = self.workRpm.rpmOnTipping;
							end
						end;
					elseif trailer.isItRunnerHookLiftTrailer then
						if self.isTipScrollerVehicle then
							if self.tipState == Trailer.TIPSTATE_OPENING or self.tipState == Trailer.TIPSTATE_OPEN or trailer:getIsAnimationPlaying("unfoldHand") then
								neededRpm = self.workRpm.rpmOnTipping;
							end
						else
							if self.tipState == Trailer.TIPSTATE_OPENING or trailer:getIsAnimationPlaying("unfoldHand") then
								neededRpm = self.workRpm.rpmOnTipping;
							end
						end;
					elseif trailer.workRpm.isOverloadVehicle then
						if self.isTipScrollerVehicle then
							if self.tipState == Trailer.TIPSTATE_OPENING or self.tipState == Trailer.TIPSTATE_OPEN  or trailer:getOverloadingActive() then
								neededRpm = self.workRpm.rpmOnTipping;
							end
						else
							if self.tipState == Trailer.TIPSTATE_OPENING or trailer:getOverloadingActive() then
								neededRpm = self.workRpm.rpmOnTipping;
							end
						end;
					elseif self.workRpm.isOverloadVehicle then
						if self:getOverloadingActive() then
							neededRpm = self.workRpm.rpmOnTipping;
						end
					else
						if self.tipState == Trailer.TIPSTATE_OPENING or trailer.tipState == Trailer.TIPSTATE_OPENING then
							neededRpm = self.workRpm.rpmOnTipping;
						end
					end;
				else
					if self.tipState ~= nil then
						if self.isTipScrollerVehicle then
							if self.tipState == Trailer.TIPSTATE_OPENING or self.tipState == Trailer.TIPSTATE_OPEN then
								neededRpm = self.workRpm.rpmOnTipping;
							end
						else
							if self.tipState == Trailer.TIPSTATE_OPENING then
								neededRpm = self.workRpm.rpmOnTipping;
							end
						end;
					elseif self.workRpm.isOverloadVehicle then
						if self:getOverloadingActive() then
							neededRpm = self.workRpm.rpmOnTipping;
						end
					end;
				end
			end
		end
		--		implements movingTools		--
		if neededRpm == 0 then
			if self.attachedImplements[1] ~= nil and self.attachedImplements[1] ~= 0 then
				local obj = self.attachedImplements[1].object;
				tcount = 0;
				if obj.movingTools ~= 0 and obj.movingTools ~= nil then
					for _ in pairs(obj.movingTools) do tcount = tcount + 1 end
				end
				if tcount ~= 0 then
					for _, toolId in ipairs(self.workRpm.movingToolIds) do
						if toolId > tcount then break; end;
						local tool = obj.movingTools[toolId];
						if tool.rotSpeed ~= nil then
							if tool.lastRotHistoryRPM == nil then
								tool.lastRotHistoryRPM = tool.curRot[tool.rotationAxis];
							end;
							if 0.00001 < math.abs(tool.lastRotHistoryRPM - tool.curRot[tool.rotationAxis]) then
								tool.lastRotHistoryRPM = tool.curRot[tool.rotationAxis];
								neededRpm = self.workRpm.rpmOnLoad;
							end;
						end;
						if tool.transSpeed ~= nil then
							if tool.lastTransHistoryRPM == nil then
								tool.lastTransHistoryRPM = tool.curTrans[tool.translationAxis];
							end;
							
							if 0.00001 < math.abs(tool.lastTransHistoryRPM - tool.curTrans[tool.translationAxis]) then
								tool.lastTransHistoryRPM = tool.curTrans[tool.translationAxis];
								neededRpm = self.workRpm.rpmOnLoad;
							end;
						end;
					end;
				end;
			end;
		end;
		--		movingTools		--
		if neededRpm == 0 then
			tcount = 0;
			if self.movingTools ~= 0 and self.movingTools ~= nil then
				for _ in pairs(self.movingTools) do tcount = tcount + 1 end
			end;

			for _, toolId in ipairs(self.workRpm.movingToolIds) do
				if toolId > tcount then break; end;
				local tool = self.movingTools[toolId];
				if tool.rotSpeed ~= nil then
					if tool.lastRotHistoryRPM == nil then
						tool.lastRotHistoryRPM = tool.curRot[tool.rotationAxis];
					end;
					if 0.00001 < math.abs(tool.lastRotHistoryRPM - tool.curRot[tool.rotationAxis]) and self.tipState ~= 3 then
						tool.lastRotHistoryRPM = tool.curRot[tool.rotationAxis];
						neededRpm = self.workRpm.rpmOnLoad;
					end;
				end;
				if tool.transSpeed ~= nil then
					if tool.lastTransHistoryRPM == nil then
						tool.lastTransHistoryRPM = tool.curTrans[tool.translationAxis];
					end;
					if 0.00001 < math.abs(tool.lastTransHistoryRPM - tool.curTrans[tool.translationAxis]) and self.tipState ~= 3 then
						tool.lastTransHistoryRPM = tool.curTrans[tool.translationAxis];
						neededRpm = self.workRpm.rpmOnLoad;
					end;
				end;
			end;
		end;
		--		Pipe folding / unfolding		--
		if self.hasPipe then
			if self.pipeCurrentState == 0 then
				neededRpm = self.workRpm.rpmFolding;
			end
		end
		--		--
		local curRpm = 0
		if SpecializationUtil.hasSpecialization(Drivable, self.specializations) and SpecializationUtil.hasSpecialization(Motorized, self.specializations) then
			curRpm = self.motor.lastMotorRpm
		else
			if self.attacherVehicle.isMotorStarted then
				curRpm = self.attacherVehicle.motor.lastMotorRpm
			end
		end
		local throttleOff = false
		if InputBinding.AXIS_ACCELERATE_VEHICLE ~= nil and InputBinding.AXIS_BRAKE_VEHICLE ~= nil then
			if InputBinding.getInputAxis(InputBinding.AXIS_ACCELERATE_VEHICLE) == 1 and InputBinding.getInputAxis(InputBinding.AXIS_BRAKE_VEHICLE) == 1 then 
				throttleOff = true
			else
				throttleOff = false
			end
		else
			throttleOff = true
		end
		if neededRpm > 0 and throttleOff == true then
			self.workRpm.rpm = neededRpm;
			self.workRpm.releaseRpmTimer = g_currentMission.time + self.workRpm.maxTime;
		else
			if g_currentMission.time > self.workRpm.releaseRpmTimer then
				self.workRpm.rpm = 0;
				self.workRpm.releaseRpmTimer = 0;
			end;
		end;
	end;
end;

function WorkRPM:draw()
	self.workRpm.showRPMHudOverlay:render();
	if self:getIsActive() and SpecializationUtil.hasSpecialization(Drivable, self.specializations) and self.attacherVehicle == nil then
		if self.isMotorStarted == false then
			setTextBold(true);
			setTextAlignment(RenderText.ALIGN_RIGHT);
			setTextColor(1, 0, 0, 1);
			renderText(self.workRpm.showRPMPosX + 0.034, self.workRpm.showRPMPosY + 0.006, 0.016, "OFF");
		end;
		if self.isMotorStarted == true then	
			if self.motor.lastMotorRpm > 1900 then
				setTextBold(true);
				setTextAlignment(RenderText.ALIGN_RIGHT);
				setTextColor(1, 0, 0, 1);
				renderText(self.workRpm.showRPMPosX + 0.034, self.workRpm.showRPMPosY + 0.006, 0.016, tostring(math.floor(self.motor.lastMotorRpm)));
			else
				setTextBold(true);
				setTextAlignment(RenderText.ALIGN_RIGHT);
				setTextColor(0, 1, 0, 1);
				renderText(self.workRpm.showRPMPosX + 0.034, self.workRpm.showRPMPosY + 0.006, 0.016, tostring(math.floor(self.motor.lastMotorRpm)));
			end;
		end;
	end;
end;

function WorkRPM:onLeave()
	self.workRpm.rpm = 0;
	self.workRpm.releaseRpmTimer = 0;
end;

function WorkRPM:getPtoRpm(oldFunc)
	return self.workRpm.rpm;
end;

function WorkRPM:getDoConsumePtoPower(oldFunc)
	return self.workRpm.rpm > 0;
end;